package com.fjhx.rabbitmq.busi;

import com.alibaba.fastjson.JSONObject;
import com.rabbitmq.client.Channel;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.listener.api.ChannelAwareMessageListener;
import org.springframework.retry.annotation.Backoff;
import org.springframework.retry.annotation.EnableRetry;
import org.springframework.retry.annotation.Retryable;
import org.springframework.stereotype.Component;

/**
 * rabbitmq队列监听工具
 */
@EnableRetry
@Component
public class BusiRabbitMQListener implements ChannelAwareMessageListener {

    @RabbitHandler
    @RabbitListener(
            queues = {
                    "rfid1_to_rfid2_queue"
            },
            concurrency = "8"
    )
    @Retryable(value = {Exception.class}, maxAttempts = 3, backoff = @Backoff(delay = 3000, multiplier = 1))
    @Override
    public void onMessage(Message message, Channel channel) throws Exception {
        try {
            String msg = new String(message.getBody(), "UTF-8");
            JSONObject jsonObject = JSONObject.parseObject(msg);

            System.err.println(jsonObject.toJSONString());

            // 手动签收消息,通知mq服务器端删除该消息
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            e.printStackTrace();
            // 丢弃该消息
            channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, false);
            throw e;
        }
    }

}

