package com.fjhx.rabbitmq.config;

import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.TopicExchange;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @Description: rabbitmq配置
 * @ClassName: RabbitMQConfig
 * @Author: linqt
 * @Date: 2021/11/9 14:39
 * @Version: 1.0
 */
@Configuration
public class RabbitMQConfig {

    @Value("${spring.rabbitmq.send.socket.queue}")
    public String queue;

    @Value("${spring.rabbitmq.send.socket.exchange}")
    public String exchange;

    @Value("${spring.rabbitmq.send.socket.routingKey}")
    public String routingKey;

    /**
     * rabbitMq里初始化队列queue.
     *
     * @return
     */
    @Bean
    public Queue queue() {
        return new Queue(queue);
    }

    /**
     * rabbitMq里初始化exchange.
     *
     * @return
     */
    @Bean
    public TopicExchange exchange() {
        return new TopicExchange(exchange);
    }

    /**
     * 绑定exchange & queue & routekey.
     *
     * @return
     */
    @Bean
    public Binding bindingExchange() {
        return BindingBuilder.bind(queue()).to(exchange()).with(routingKey);
    }
}
