package com.fjhx.rfid.r2000.sd.service.impl;

import com.fjhx.config.RFIDConfig;
import com.fjhx.entity.rfid.RFIDClient;
import com.fjhx.rfid.r2000.sd.service.ISdService;
import com.fjhx.rfid.r2000.tcp.service.ISdTcpService;
import com.fjhx.rfid.r2000.udp.service.ISdUdpService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @ClassName: TcpServiceImpl
 * @Author: linqt
 * @Date: 2022/3/17 10:32
 * @Version: 1.0
 */
@Service
public class SdServiceImpl implements ISdService {

    @Autowired
    private ISdTcpService iSdTcpService;

    @Autowired
    private ISdUdpService iSdUdpService;

    /**
     * 启动
     *
     * @param client 参数
     * @return
     */
    @Override
    public void open(RFIDClient client) {
        //传进来的ip集合
        List<String> ips = client.getIpList();
        //默认tcp协议ip集合
        List<String> tcpIps = RFIDConfig.getSd().getTcp().getIps();
        //默认udp协议ip集合
        List<String> updIps = RFIDConfig.getSd().getUdp().getIps();
        if (CollectionUtils.isNotEmpty(ips)) {
            List<String> finalTcpIps = tcpIps;
            tcpIps = ips.stream()
                    .filter(item -> finalTcpIps.stream().collect(Collectors.toList())
                            .contains(item)).collect(Collectors.toList());

            List<String> finalUpdIps = updIps;
            updIps = ips.stream()
                    .filter(item -> finalUpdIps.stream().collect(Collectors.toList())
                            .contains(item)).collect(Collectors.toList());
        }

        if (CollectionUtils.isNotEmpty(tcpIps)) {
            //启动tcp读写器
            iSdTcpService.open(tcpIps);
        }

        if (CollectionUtils.isNotEmpty(updIps)) {
            //启动upd读写器
            iSdUdpService.open(updIps);
        }
    }

    /**
     * 关闭
     *
     * @param client 参数
     * @return
     */
    @Override
    public void close(RFIDClient client) {
        //传进来的ip集合
        List<String> ips = client.getIpList();
        //默认tcp协议ip集合
        List<String> tcpIps = RFIDConfig.getSd().getTcp().getIps();
        //默认udp协议ip集合
        List<String> updIps = RFIDConfig.getSd().getUdp().getIps();
        if (CollectionUtils.isNotEmpty(ips)) {
            List<String> finalTcpIps = tcpIps;
            tcpIps = ips.stream()
                    .filter(item -> finalTcpIps.stream().collect(Collectors.toList())
                            .contains(item)).collect(Collectors.toList());

            List<String> finalUpdIps = updIps;
            updIps = ips.stream()
                    .filter(item -> finalUpdIps.stream().collect(Collectors.toList())
                            .contains(item)).collect(Collectors.toList());
        }

        if (CollectionUtils.isNotEmpty(tcpIps)) {
            //关闭tcp读写器
            iSdTcpService.close(tcpIps);
        }

        if (CollectionUtils.isNotEmpty(updIps)) {
            //关闭udp读写器
            iSdUdpService.close(updIps);
        }
    }
}
