package com.fjhx.rfid.r2000.tcp.callback;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.config.RFIDConfig;
import com.fjhx.constant.RFIDConstant;
import com.fjhx.constant.RedisKeyConstant;
import com.fjhx.rabbitmq.service.RabbitmqService;
import com.fjhx.redis.RedisCache;
import com.fjhx.utils.id.IdUtils;
import com.fjhx.utils.spring.SpringUtils;
import com.rfid.callBack.CallBack.R2000;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.text.MessageFormat;
import java.util.Date;
import java.util.concurrent.TimeUnit;

@Data
public class SdTcpCallBack implements R2000 {

    /**
     * redis实例
     */
    private RedisCache redisCache = SpringUtils.getBean(RedisCache.class);

    /**
     * rabbitmq实例
     */
    private RabbitmqService rabbitmqService = SpringUtils.getBean(RabbitmqService.class);

    private String ip;

    public SdTcpCallBack(String ip) {
        this.ip = ip;
    }

    public void readData(String data, String rssi, String antennaNo, String deviceNo, String direction, String communicationMode) {
        try {
            if (StringUtils.isBlank(data)) {
                return;
            }

            //是否发送
            Boolean isSend = false;

            //获取rfid缓存信息
            Long time = redisCache.getCacheObject(MessageFormat.format(RedisKeyConstant.RFID_REDIS_CACHE_PREFIX, ip + "-" + antennaNo + "-" + data));
            //缓存记录为空，或者上次的记录时间离现在超过5s，则发送信息
            if (time == null || System.currentTimeMillis() - time > RFIDConstant.DEFAULT_TIME) {
                isSend = true;
            }

            // 存入缓存
            redisCache.setCacheObject(MessageFormat.format(RedisKeyConstant.RFID_REDIS_CACHE_PREFIX, ip + "-" + antennaNo + "-" + data), new Date().getTime(), RFIDConfig.getDefaultTime(), TimeUnit.MILLISECONDS);

            // 封装实际数据
            JSONObject object = new JSONObject();
            object.put("RFID", data);
            object.put("ANTNO", antennaNo);
            object.put("IP", ip);

            if (isSend) {
                // -------------------- 使用rabbitmq派转消息 --------------------
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("messageId", IdUtils.fastSimpleUUID());
                jsonObject.put("rfidMsg", object);
                rabbitmqService.sdSend(jsonObject);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void onceReadTagEnd(Boolean result) {

    }

    public void serialPortException(String name, boolean result) {

    }

    public void heartBeat(String value, String deviceNo, String communicationMode) {
        System.out.println("value " + value + " deviceNo " + deviceNo + " communicationMode " + communicationMode);
    }

    public void infraredTrigger(boolean direction, String value, String deviceNo, String communicationMode) {
        System.out.println("方向 " + direction + " 红外值 " + value + " 设备号 " + deviceNo + " IP或串口号 " + communicationMode);
    }

    public void onceReadTagEnd(boolean arg0) {

    }
}