package com.fjhx.rfid.r2000.tcp.service.impl;

import com.fjhx.config.RFIDConfig;
import com.fjhx.constant.RFIDConstant;
import com.fjhx.rfid.r2000.tcp.callback.SdTcpCallBack;
import com.fjhx.rfid.r2000.tcp.service.ISdTcpService;
import com.fjhx.thread.ThreadPoolManager;
import com.fjhx.utils.AjaxResult;
import com.fjhx.utils.StringUtils;
import com.fjhx.web.exception.BaseException;
import com.rfid.uhf.controller.impl.ReaderR2000;
import com.rfid.uhf.service.ReaderR2000Service;
import com.rfid.uhf.service.impl.ReaderR2000ServiceImpl;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;

import java.text.MessageFormat;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CountDownLatch;

/**
 * @Description:
 * @ClassName: TcpServiceImpl
 * @Author: linqt
 * @Date: 2022/3/17 10:32
 * @Version: 1.0
 */
@Service
public class ISdTcpServiceImpl implements ISdTcpService {

    /**
     * 初始化实例集合
     */
    private static ConcurrentHashMap<String, ConcurrentHashMap<String, Object>> sdTcpServerMap = new ConcurrentHashMap<>();

    /**
     * 启动读写器
     *
     * @param ips 读写器ip集合
     * @return
     */
    @Override
    public AjaxResult open(List<String> ips) {
        if (CollectionUtils.isEmpty(ips)) {
            throw new BaseException("读写器IP地址不能为空！");
        }
        try {
            CountDownLatch countDownLatch = new CountDownLatch(ips.size());
            // 启动扫描线程
            for (String ip : ips) {
                //如果读写器已开启，则跳过
                if (ObjectUtils.isNotEmpty(sdTcpServerMap.get(ip))) {
                    System.err.println("IP为【" + ip + "】的读写器已开启。");
                    continue;
                }

                CountDownLatch errCountDownLatch = countDownLatch;
                ThreadPoolManager.execute(() -> {
                    try {
                        ReaderR2000Service service = new ReaderR2000ServiceImpl();
                        ReaderR2000 reader = service.connect(ip, RFIDConfig.getSd().getTcp().getPort(), new SdTcpCallBack(ip));

                        // 实例存储
                        ConcurrentHashMap<String, Object> sMap = new ConcurrentHashMap<>();
                        sMap.put("service", service);
                        sMap.put("reader", reader);
                        sdTcpServerMap.put(ip, sMap);
                        System.err.println("连接成功----------");
                        // 开始扫描
                        service.beginInvV2(reader);
                        while (true){
                            String v = service.version(reader);
                            if(ObjectUtils.isEmpty(v)){
                                sdTcpServerMap.clear();
                                this.open(ips);
                            }else{
                                System.err.println("当前连接正常----设备版本为："+v);
                                Thread.sleep(3000);
                            }
                        }
                    } catch (Exception e) {
//                        e.printStackTrace();
//                        if (errCountDownLatch != null) {
//                            errCountDownLatch.countDown();
//                        }
//                        throw new BaseException(MessageFormat.format(RFIDConstant.SERVER_START_ERR, ip));
                        try {
                            System.err.println("重新连接中----------");
                            Thread.sleep(3000);
                            this.open(ips);
                        } catch (InterruptedException ex) {
                            ex.printStackTrace();
                        }
                    }
                });
            }

            if (countDownLatch != null) {
                countDownLatch.await();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return AjaxResult.success();
    }

    /**
     * 关闭读写器
     *
     * @param ips 读写器ip集合
     * @return
     */
    @Override
    public AjaxResult close(List<String> ips) {
        if (CollectionUtils.isEmpty(ips)) {
            throw new BaseException("参数缺失！");
        }
        // 关闭线程
        for (String ip : ips) {
            try {
                //如果读写器已开启，则跳过
                if (ObjectUtils.isEmpty(sdTcpServerMap.get(ip))) {
                    System.err.println("IP为【" + ip + "】的读写器已关闭。");
                    continue;
                }

                // 获取实例
                ConcurrentHashMap<String, Object> map = sdTcpServerMap.get(ip);
                ReaderR2000Service service = (ReaderR2000Service) map.get("service");
                ReaderR2000 reader = (ReaderR2000) map.get("reader");

                // 结束扫描
                service.stopInvV2(reader);

                // 关闭连接
                service.disconnect(reader);

                // 移除服务
                sdTcpServerMap.remove(ip);
            } catch (Exception e) {
                e.printStackTrace();
                throw new BaseException(MessageFormat.format(RFIDConstant.SERVER_CLOSE_ERR, ip));
            }
        }

        return AjaxResult.success();
    }
}
