package com.fjhx.rfid.r2000.udp.callback;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.config.RFIDConfig;
import com.fjhx.constant.RFIDConstant;
import com.fjhx.constant.RedisKeyConstant;
import com.fjhx.rabbitmq.service.RabbitmqService;
import com.fjhx.redis.RedisCache;
import com.fjhx.rfid.r2000.udp.utils.Utility;
import com.fjhx.utils.StringUtils;
import com.fjhx.utils.id.IdUtils;
import com.fjhx.utils.spring.SpringUtils;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.socket.DatagramPacket;
import lombok.Data;

import java.text.MessageFormat;
import java.util.Date;
import java.util.concurrent.TimeUnit;

@Data
public class SdUdpCallBack extends SimpleChannelInboundHandler<DatagramPacket> {

    /**
     * redis实例
     */
    private RedisCache redisCache = SpringUtils.getBean(RedisCache.class);

    /**
     * rabbitmq实例
     */
    private RabbitmqService rabbitmqService = SpringUtils.getBean(RabbitmqService.class);

    private String ip;

    public SdUdpCallBack(String ip) {
        this.ip = ip;
    }

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, DatagramPacket msg) {
        try {
            ByteBuf byteBuf = msg.content();
            byte[] bytes = new byte[byteBuf.readableBytes()];
            byteBuf.readBytes(bytes);
            String str = Utility.bytes2HexString(bytes);

            // 如果返回的结果长度小于20，则跳过
            if (StringUtils.isNotBlank(str) && str.length() > 20) {
                // 截取天线号
                String antNo = str.substring(RFIDConstant.UDP_RESULT_DATA_SUBSTRING_ANT_NO_BEGIN_INDEX, RFIDConstant.UDP_RESULT_DATA_SUBSTRING_ANT_NO_END_INDEX);
                // 截取rfid
                String rfid = str.substring(RFIDConstant.UDP_RESULT_DATA_SUBSTRING_RFID_BEGIN_INDEX, str.length() - RFIDConstant.UDP_RESULT_DATA_SUBSTRING_RFID_END_INDEX);

                //是否发送
                Boolean isSend = false;

                //获取rfid缓存信息
                Long time = redisCache.getCacheObject(MessageFormat.format(RedisKeyConstant.RFID_REDIS_CACHE_PREFIX, ip + "-" + antNo + "-" + rfid));
                //缓存记录为空，或者上次的记录时间离现在超过5s，则发送信息
                if (time == null || System.currentTimeMillis() - time > RFIDConstant.DEFAULT_TIME) {
                    isSend = true;
                }

                // 存入缓存
                redisCache.setCacheObject(MessageFormat.format(RedisKeyConstant.RFID_REDIS_CACHE_PREFIX, ip + "-" + antNo + "-" + rfid), new Date().getTime(), RFIDConfig.getDefaultTime(), TimeUnit.MILLISECONDS);

                // 封装实际数据
                JSONObject object = new JSONObject();
                object.put("RFID", rfid);
                object.put("ANTNO", antNo);
                object.put("IP", ip);

                if (isSend) {
                    // -------------------- 使用rabbitmq派转消息 --------------------
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("messageId", IdUtils.fastSimpleUUID());
                    jsonObject.put("rfidMsg", object);
                    rabbitmqService.sdSend(jsonObject);
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}